@@ -41,6 +41,10 @@ import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
 
 import static org.xnio._private.Messages.msg;
 
+import org.wildfly.common.ref.CleanerReference;
+import org.wildfly.common.ref.Reaper;
+import org.wildfly.common.ref.Reference;
+
 /**
  * Buffer utility methods.
  *
@@ -1947,6 +1951,55 @@ public final class Buffers {
         };
     }
 
+    /**
+     * Create a pooled wrapper around a buffer that was allocated via {@link ByteBufferPool}.  The buffer is freed to the
+     * global pool when freed.
+     *
+     * @param buffer the buffer to wrap
+     * @return the pooled wrapper
+     */
+    public static Pooled<ByteBuffer> globalPooledWrapper(final ByteBuffer buffer) {
+        return new Pooled<ByteBuffer>() {
+            private volatile ByteBuffer buf = buffer;
+
+            public void discard() {
+                ByteBuffer oldBuf = this.buf;
+                if (oldBuf == null) return;
+                final ByteBuffer buf = oldBuf.duplicate();
+                new CleanerReference<ByteBuffer, Void>(this.buf, null, new Reaper<ByteBuffer, Void>() {
+                    public void reap(final Reference<ByteBuffer, Void> reference) {
+                        // free the duplicate
+                        ByteBufferPool.free(buf);
+                    }
+                });
+                this.buf = null;
+            }
+
+            public void free() {
+                ByteBuffer oldBuf = this.buf;
+                if (oldBuf == null) return;
+                ByteBufferPool.free(oldBuf);
+                buf = null;
+            }
+
+            public ByteBuffer getResource() throws IllegalStateException {
+                final ByteBuffer buffer = buf;
+                if (buffer == null) {
+                    throw new IllegalStateException();
+                }
+                return buffer;
+            }
+
+            public void close() {
+                free();
+            }
+
+            public String toString() {
+                return "Globally pooled wrapper around " + buffer;
+            }
+        };
+    }
+
     /**
      * Create a "pooled" empty buffer.  Discarding or freeing the buffer has no effect; the returned buffer is
      * always empty.
@@ -2215,6 +2268,35 @@ public final class Buffers {
         }
     }
 
+    /**
+     * Get a debug-friendly description of the buffer.
+     *
+     * @param buffer the buffer to describe
+     * @return the string
+     */
+    public static String debugString(ByteBuffer buffer) {
+        StringBuilder b = new StringBuilder();
+        b.append("1 buffer of ").append(buffer.remaining()).append(" bytes");
+        return b.toString();
+    }
+
+    /**
+     * Get a debug-friendly description of the buffer.
+     *
+     * @param buffers the buffers to describe
+     * @param offs the offset into the array
+     * @param len the number of buffers
+     * @return the string
+     */
+    public static String debugString(ByteBuffer[] buffers, int offs, int len) {
+        StringBuilder b = new StringBuilder();
+        b.append(len).append(" buffer(s)");
+        if (len > 0) {
+            b.append(" of ").append(Buffers.remaining(buffers, offs, len)).append(" bytes");
+        }
+        return b.toString();
+    }
+
     /**
      * Empty a buffer to an output stream.  Specially optimized for heap buffers.  If a partial transfer occurs
      * due to interruption, the buffer's position is updated accordingly.
