@@ -910,7 +910,7 @@ public final class Channels {
             if (count == 0L) return total;
             if (NULL_FILE_CHANNEL != null) {
                 while (count > 0) {
-                    if ((lres = channel.transferTo(0, count, NULL_FILE_CHANNEL)) == 0L) {
+                    if ((lres = channel.transferTo(0, count, NULL_FILE_CHANNEL)) <= 0L) {
                         break;
                     }
                     total += lres;
@@ -950,7 +950,7 @@ public final class Channels {
                 if (count == 0L) return total;
                 if (NULL_FILE_CHANNEL != null) {
                     while (count > 0) {
-                        if ((lres = NULL_FILE_CHANNEL.transferFrom(channel, 0, count)) == 0L) {
+                        if ((lres = NULL_FILE_CHANNEL.transferFrom(channel, 0, count)) <= 0L) {
                             break;
                         }
                         total += lres;
@@ -983,34 +983,31 @@ public final class Channels {
      * @throws IOException if an error occurs
      */
     public static long drain(FileChannel channel, long position, long count) throws IOException {
-        if (channel instanceof StreamSourceChannel) {
-            return drain((StreamSourceChannel) channel, count);
-        } else {
-            long total = 0L, lres;
-            int ires;
-            ByteBuffer buffer = null;
-            for (;;) {
-                if (count == 0L) return total;
-                if (NULL_FILE_CHANNEL != null) {
-                    while (count > 0) {
-                        if ((lres = channel.transferTo(position, count, NULL_FILE_CHANNEL)) == 0L) {
-                            break;
-                        }
-                        total += lres;
-                        count -= lres;
+        long total = 0L, lres;
+        int ires;
+        ByteBuffer buffer = null;
+        for (;;) {
+            if (count == 0L) return total;
+            if (NULL_FILE_CHANNEL != null) {
+                while (count > 0) {
+                    if ((lres = channel.transferTo(position, count, NULL_FILE_CHANNEL)) <= 0L) {
+                        break;
                     }
-                    // jump out quick if we drained the fast way
-                    if (total > 0L) return total;
-                }
-                if (buffer == null) buffer = DRAIN_BUFFER.duplicate();
-                if ((long) buffer.limit() > count) buffer.limit((int) count);
-                ires = channel.read(buffer);
-                buffer.clear();
-                switch (ires) {
-                    case -1: return total == 0L ? -1L : total;
-                    case 0: return total;
-                    default: total += (long) ires;
+                    total += lres;
+                    count -= lres;
+                    position += lres;
                 }
+                // jump out quick if we drained the fast way
+                if (total > 0L) return total;
+            }
+            if (buffer == null) buffer = DRAIN_BUFFER.duplicate();
+            if ((long) buffer.limit() > count) buffer.limit((int) count);
+            ires = channel.read(buffer, position);
+            buffer.clear();
+            switch (ires) {
+                case -1: return total == 0L ? -1L : total;
+                case 0: return total;
+                default: total += (long) ires;
             }
         }
     }
@@ -1095,7 +1092,7 @@ public final class Channels {
                 final String osName = System.getProperty("os.name", "unknown").toLowerCase(Locale.US);
                 try {
                     if (osName.contains("windows")) {
-                        return new FileOutputStream("NUL:").getChannel();
+                        return new FileOutputStream("NUL").getChannel();
                     } else {
                         return new FileOutputStream("/dev/null").getChannel();
                     }
