@@ -19,24 +19,10 @@
 
 package org.xnio.channels;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertSame;
-import static org.junit.Assert.assertTrue;
-import static org.xnio.AssertReadWrite.assertReadMessage;
-import static org.xnio.AssertReadWrite.assertWrittenMessage;
-
-import java.io.File;
-import java.io.IOException;
-import java.io.RandomAccessFile;
-import java.nio.ByteBuffer;
-import java.nio.channels.FileChannel;
-import java.util.concurrent.TimeUnit;
-
 import org.junit.Before;
+import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.TemporaryFolder;
 import org.xnio.Buffers;
 import org.xnio.ChannelListener;
 import org.xnio.Option;
@@ -45,6 +31,31 @@ import org.xnio.StreamConnection;
 import org.xnio.mock.AcceptingChannelMock;
 import org.xnio.mock.ConnectedStreamChannelMock;
 import org.xnio.mock.MessageChannelMock;
+import org.xnio.mock.ReadableByteChannelMock;
+
+import java.io.BufferedWriter;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileWriter;
+import java.io.IOException;
+import java.io.RandomAccessFile;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
+import java.nio.ByteBuffer;
+import java.nio.channels.ClosedChannelException;
+import java.nio.channels.FileChannel;
+import java.util.concurrent.TimeUnit;
+import java.util.function.LongFunction;
+
+import static java.nio.charset.StandardCharsets.UTF_8;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
+import static org.xnio.AssertReadWrite.assertReadMessage;
+import static org.xnio.AssertReadWrite.assertWrittenMessage;
 
 /**
  * Test for {@link Channels}.
@@ -69,7 +80,7 @@ public class ChannelsTestCase {
         assertTrue(connectedChannelMock.isFlushed());
         connectedChannelMock.enableFlush(false);
         ByteBuffer buffer = ByteBuffer.allocate(10);
-        buffer.put("10".getBytes("UTF-8")).flip();
+        buffer.put("10".getBytes(UTF_8)).flip();
         assertEquals(2, connectedChannelMock.write(buffer));
         assertWrittenMessage(connectedChannelMock, "10");
         assertFalse(connectedChannelMock.isFlushed());
@@ -88,7 +99,7 @@ public class ChannelsTestCase {
     public void shutdownWritesBlocking() throws IOException, InterruptedException {
         connectedChannelMock.enableFlush(false);
         ByteBuffer buffer = ByteBuffer.allocate(10);
-        buffer.put("shutdown".getBytes("UTF-8")).flip();
+        buffer.put("shutdown".getBytes(UTF_8)).flip();
         assertEquals(8, connectedChannelMock.write(buffer));
         assertWrittenMessage(connectedChannelMock, "shutdown");
         assertFalse(connectedChannelMock.isShutdownWrites());
@@ -107,7 +118,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void writeBlocking() throws IOException, InterruptedException {
+    public void writeBlocking() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         WriteBlocking writeRunnable = new WriteBlocking(connectedChannelMock, "write this");
         Thread writeThread = new Thread(writeRunnable);
@@ -123,7 +134,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void writeBlockingWithTimeout() throws IOException, InterruptedException {
+    public void writeBlockingWithTimeout() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         WriteBlocking writeRunnable = new WriteBlocking(connectedChannelMock, "write with timeout", 1000, TimeUnit.MICROSECONDS);
         Thread writeThread = new Thread(writeRunnable);
@@ -139,7 +150,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void writeBufferArrayBlocking() throws IOException, InterruptedException {
+    public void writeBufferArrayBlocking() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         WriteBufferArrayBlocking writeRunnable = new WriteBufferArrayBlocking(connectedChannelMock, "write", " this");
         Thread writeThread = new Thread(writeRunnable);
@@ -155,7 +166,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void writeBufferArrayBlockingWithTimeout() throws IOException, InterruptedException {
+    public void writeBufferArrayBlockingWithTimeout() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         WriteBufferArrayBlocking writeRunnable = new WriteBufferArrayBlocking(connectedChannelMock, 1000,
                 TimeUnit.MILLISECONDS, "write", "with", "timeout");
@@ -172,7 +183,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void sendBlocking() throws IOException, InterruptedException {
+    public void sendBlocking() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         SendBlocking sendRunnable = new SendBlocking(messageChannelMock, "send this");
         Thread sendThread = new Thread(sendRunnable);
@@ -187,7 +198,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void sendBlockingWithTimeout() throws IOException, InterruptedException {
+    public void sendBlockingWithTimeout() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         SendBlocking sendRunnable = new SendBlocking(messageChannelMock, "send with timeout", 1000, TimeUnit.MICROSECONDS);
         Thread sendThread = new Thread(sendRunnable);
@@ -204,7 +215,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void sendBufferArrayBlocking() throws IOException, InterruptedException {
+    public void sendBufferArrayBlocking() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         SendBufferArrayBlocking sendRunnable = new SendBufferArrayBlocking(messageChannelMock, "send", " this");
         Thread sendThread = new Thread(sendRunnable);
@@ -219,7 +230,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void sendBufferArrayBlockingWithTimeout() throws IOException, InterruptedException {
+    public void sendBufferArrayBlockingWithTimeout() throws InterruptedException {
         connectedChannelMock.enableWrite(false);
         SendBufferArrayBlocking sendRunnable = new SendBufferArrayBlocking(messageChannelMock, 1000,
                 TimeUnit.MILLISECONDS, "send", "with", "timeout");
@@ -237,7 +248,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void readBlocking() throws IOException, InterruptedException {
+    public void readBlocking() throws InterruptedException {
         connectedChannelMock.setReadData("read this");
         ReadBlocking readRunnable = new ReadBlocking(connectedChannelMock);
         Thread readThread = new Thread(readRunnable);
@@ -253,7 +264,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void readBlockingToEmptyBuffer() throws IOException, InterruptedException {
+    public void readBlockingToEmptyBuffer() throws InterruptedException {
         connectedChannelMock.setReadData("can't read this");
         ReadBlocking readRunnable = new ReadBlocking(connectedChannelMock, Buffers.EMPTY_BYTE_BUFFER);
         Thread readThread = new Thread(readRunnable);
@@ -264,7 +275,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void readBlockingWithTimeout() throws IOException, InterruptedException {
+    public void readBlockingWithTimeout() throws InterruptedException {
         connectedChannelMock.setReadData("read with timeout");
         ReadBlocking readRunnable = new ReadBlocking(connectedChannelMock, 100, TimeUnit.MILLISECONDS);
         Thread readThread = new Thread(readRunnable);
@@ -280,7 +291,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void readBlockingWithTimeoutToEmptyBuffer() throws IOException, InterruptedException {
+    public void readBlockingWithTimeoutToEmptyBuffer() throws InterruptedException {
         connectedChannelMock.setReadData("can't read this");
         ReadBlocking readRunnable = new ReadBlocking(connectedChannelMock, 100, TimeUnit.MILLISECONDS, Buffers.EMPTY_BYTE_BUFFER);
         Thread readThread = new Thread(readRunnable);
@@ -295,7 +306,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void readBlockingToBufferArray() throws IOException, InterruptedException {
+    public void readBlockingToBufferArray() throws InterruptedException {
         connectedChannelMock.setReadData("read", "this");
         ReadToBufferArrayBlocking readRunnable = new ReadToBufferArrayBlocking(connectedChannelMock);
         Thread readThread = new Thread(readRunnable);
@@ -315,7 +326,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void readBlockingToBufferArrayWithTimeout() throws IOException, InterruptedException {
+    public void readBlockingToBufferArrayWithTimeout() throws InterruptedException {
         connectedChannelMock.setReadData("read", "with", "timeout");
         ReadToBufferArrayBlocking readRunnable = new ReadToBufferArrayBlocking(connectedChannelMock, 1000,
                 TimeUnit.MILLISECONDS);
@@ -336,13 +347,13 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void readBlockingToEmptyBufferArrayWithTimeout() throws IOException, InterruptedException {
+    public void readBlockingToEmptyBufferArrayWithTimeout() throws IOException {
         connectedChannelMock.setReadData("can't read this");
         assertEquals(0, Channels.readBlocking(connectedChannelMock, new ByteBuffer[0], 0, 0, 2, TimeUnit.MINUTES));
     }
 
     @Test
-    public void receiveBlocking() throws IOException, InterruptedException {
+    public void receiveBlocking() throws InterruptedException {
         connectedChannelMock.setReadData("receive this");
         ReceiveBlocking receiveRunnable = new ReceiveBlocking(messageChannelMock);
         Thread receiveThread = new Thread(receiveRunnable);
@@ -358,7 +369,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void receiveBlockingWithTimeout() throws IOException, InterruptedException {
+    public void receiveBlockingWithTimeout() throws InterruptedException {
         connectedChannelMock.setReadData("receive with timeout");
         ReceiveBlocking receiveRunnable = new ReceiveBlocking(messageChannelMock, 100, TimeUnit.MILLISECONDS);
         Thread receiveThread = new Thread(receiveRunnable);
@@ -374,7 +385,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void receiveBufferArrayBlocking() throws IOException, InterruptedException {
+    public void receiveBufferArrayBlocking() throws InterruptedException {
         connectedChannelMock.setReadData("receive", "this");
         ReceiveBufferArrayBlocking receiveRunnable = new ReceiveBufferArrayBlocking(messageChannelMock);
         Thread receiveThread = new Thread(receiveRunnable);
@@ -394,7 +405,7 @@ public class ChannelsTestCase {
     }
 
     @Test
-    public void receiveBufferArrayBlockingWithTimeout() throws IOException, InterruptedException {
+    public void receiveBufferArrayBlockingWithTimeout() throws InterruptedException {
         connectedChannelMock.setReadData("receive", "with", "timeout");
         ReceiveBufferArrayBlocking receiveRunnable = new ReceiveBufferArrayBlocking(messageChannelMock, 1000,
                 TimeUnit.MILLISECONDS);
@@ -417,7 +428,7 @@ public class ChannelsTestCase {
     @Test
     public void acceptBlocking() throws IOException, InterruptedException {
         final AcceptingChannelMock acceptingChannelMock = new AcceptingChannelMock();
-        final AcceptBlocking<?> acceptBlockingRunnable = new AcceptBlocking<StreamConnection>(acceptingChannelMock);
+        final AcceptBlocking<?> acceptBlockingRunnable = new AcceptBlocking<>(acceptingChannelMock);
         final Thread acceptChannelThread = new Thread(acceptBlockingRunnable);
         assertNotNull(Channels.acceptBlocking(acceptingChannelMock));
         assertFalse(acceptingChannelMock.haveWaitedAcceptable());
@@ -437,7 +448,7 @@ public class ChannelsTestCase {
     @Test
     public void acceptBlockingWithTimeout() throws IOException, InterruptedException {
         final AcceptingChannelMock acceptingChannelMock = new AcceptingChannelMock();
-        final AcceptBlocking<?> acceptBlockingRunnable = new AcceptBlocking<StreamConnection>(acceptingChannelMock, 10, TimeUnit.SECONDS);
+        final AcceptBlocking<?> acceptBlockingRunnable = new AcceptBlocking<>(acceptingChannelMock, 10, TimeUnit.SECONDS);
         final Thread acceptChannelThread = new Thread(acceptBlockingRunnable);
         // try to accept blocking with acceptance enabled at accepting channel mock
         assertNotNull(Channels.acceptBlocking(acceptingChannelMock, 1, TimeUnit.SECONDS));
@@ -447,7 +458,7 @@ public class ChannelsTestCase {
         acceptChannelThread.start();
         acceptChannelThread.join(200);
         assertFalse(acceptChannelThread.isAlive());
-        // thread is supposed to have finished, after having invoked awaitAcceptable at acceptingchannelMock with 10s timeout
+        // thread is supposed to have finished, after having invoked awaitAcceptable at acceptingChannelMock with 10s timeout
         assertTrue(acceptingChannelMock.haveWaitedAcceptable());
         assertEquals(10, acceptingChannelMock.getAwaitAcceptableTime());
         assertEquals(TimeUnit.SECONDS, acceptingChannelMock.getAwaitAcceptableTimeUnit());
@@ -515,7 +526,7 @@ public class ChannelsTestCase {
         final FileChannel fileChannel = randomAccessFile.getChannel();
         try {
             final ByteBuffer buffer = ByteBuffer.allocate(10);
-            buffer.put("test".getBytes("UTF-8")).flip();
+            buffer.put("test".getBytes(UTF_8)).flip();
             assertEquals(4, fileChannel.write(buffer));
             fileChannel.position(0);
             Channels.transferBlocking(channelMock, fileChannel, 0, 4);
@@ -536,7 +547,7 @@ public class ChannelsTestCase {
         final FileChannel fileChannel = randomAccessFile.getChannel();
         try {
             final ByteBuffer buffer = ByteBuffer.allocate(10);
-            buffer.put("test12345".getBytes("UTF-8")).flip();
+            buffer.put("test12345".getBytes(UTF_8)).flip();
             assertEquals(9, fileChannel.write(buffer));
             fileChannel.position(0);
             
@@ -556,9 +567,7 @@ public class ChannelsTestCase {
     @Test
     public void setChannelListeners() {
         final ConnectedStreamChannelMock channelMock = new ConnectedStreamChannelMock();
-        final ChannelListener<ConnectedStreamChannel> channelListener = new ChannelListener<ConnectedStreamChannel>() {
-            public void handleEvent(final ConnectedStreamChannel channel) {}
-        };
+        final ChannelListener<ConnectedStreamChannel> channelListener = channel -> {};
 
         // test setReadListener
         Channels.setReadListener(channelMock, channelListener);
@@ -582,9 +591,7 @@ public class ChannelsTestCase {
     @Test
     public void setAcceptListener() {
         final AcceptingChannelMock channelMock = new AcceptingChannelMock();
-        final ChannelListener<AcceptingChannel<StreamConnection>> channelListener = new ChannelListener<AcceptingChannel<StreamConnection>>() {
-            public void handleEvent(final AcceptingChannel<StreamConnection> channel) {}
-        };
+        final ChannelListener<AcceptingChannel<StreamConnection>> channelListener = channel -> {};
 
         Channels.setAcceptListener(channelMock, channelListener);
         assertSame(channelListener, channelMock.getAcceptListener());
@@ -629,16 +636,16 @@ public class ChannelsTestCase {
         assertReadMessage(bufferArray[5]);
         // test write(ByteBuffer)
         buffer.clear();
-        buffer.put("write".getBytes("UTF-8")).flip();
+        buffer.put("write".getBytes(UTF_8)).flip();
         wrappedByteChannel.write(buffer);
         assertWrittenMessage(channelMock, "write");
         // test write(ByteBuffer[])
         for(ByteBuffer bufferItem: bufferArray) {
             bufferItem.clear();
         }
-        bufferArray[0].put("writ".getBytes("UTF-8")).flip();
-        bufferArray[1].put("e_ag".getBytes("UTF-8")).flip();
-        bufferArray[2].put("ain".getBytes("UTF-8")).flip();
+        bufferArray[0].put("writ".getBytes(UTF_8)).flip();
+        bufferArray[1].put("e_ag".getBytes(UTF_8)).flip();
+        bufferArray[2].put("ain".getBytes(UTF_8)).flip();
         bufferArray[3].flip();
         bufferArray[4].flip();
         bufferArray[5].flip();
@@ -692,13 +699,14 @@ public class ChannelsTestCase {
         assertEquals(1000, Channels.getOption(channelMock, Options.MAX_OUTBOUND_MESSAGE_SIZE, 1000));
         assertEquals(5000, Channels.getOption(brokenConfigurable, Options.SSL_CLIENT_SESSION_TIMEOUT, 5000));
         // long type option
-        assertEquals(1l, Channels.getOption(channelMock, Options.STACK_SIZE, 1l));
-        channelMock.setOption(Options.STACK_SIZE, 50000l);
-        assertEquals(50000l, Channels.getOption(channelMock, Options.STACK_SIZE, 100));
+        assertEquals(1L, Channels.getOption(channelMock, Options.STACK_SIZE, 1L));
+        channelMock.setOption(Options.STACK_SIZE, 50000L);
+        assertEquals(50000L, Channels.getOption(channelMock, Options.STACK_SIZE, 100));
         assertEquals(100, Channels.getOption(brokenConfigurable, Options.STACK_SIZE, 100));
     }
 
     @Test
+    @SuppressWarnings( "deprecation" )
     public void unwrap() {
         assertNull(Channels.unwrap(ConnectedStreamChannelMock.class, null));
         final ConnectedStreamChannelMock channelMock = new ConnectedStreamChannelMock();
@@ -708,6 +716,207 @@ public class ChannelsTestCase {
         assertNull(Channels.unwrap(FramedMessageChannel.class, channelMock));
     }
 
+    @Test
+    public void drainStreamSourceChannel() throws IOException {
+        ConnectedStreamChannelMock channelMock = new ConnectedStreamChannelMock();
+        assertDrain(channelMock, (long count)-> {
+            try {
+                return Channels.drain(channelMock, count);
+            } catch (IOException e) {
+                throw new RuntimeException(e);
+            }
+        });
+    }
+
+    @Test
+    public void drainStreamSourceChannelAsReadableByteChannel() throws IOException {
+        ReadableByteChannelMock channelMock = new ConnectedStreamChannelMock();
+        assertDrain(channelMock, (long count)-> {
+            try {
+                return Channels.drain(channelMock, count);
+            } catch (IOException e) {
+                throw new RuntimeException(e);
+            }
+        });
+    }
+
+    @Test
+    public void drainReadableByteChannel() throws IOException {
+        ReadableByteChannelMock channelMock = new ReadableByteChannelMock();
+        assertDrain(channelMock, (long count)-> {
+            try {
+                return Channels.drain(channelMock, count);
+            } catch (IOException e) {
+                throw new RuntimeException(e);
+            }
+        });
+    }
+
+    @Rule
+    public TemporaryFolder folder = new TemporaryFolder();
+
+    @Test
+    public void drainFileChannel() throws IOException {
+        final File file1 = folder.newFile();
+        //write out data to the test files
+        final FileWriter fw1 = new FileWriter( file1 );
+        final BufferedWriter bw1 = new BufferedWriter( fw1 );
+        for (int i = 0; i < 5; i++)
+            bw1.write( "read data\n");
+        bw1.close();
+        fw1.close();
+        for (Method method: FileChannel.class.getDeclaredMethods())
+            if (Modifier.isStatic(method.getModifiers()))
+                System.out.println("method: " + method);
+        final FileChannel fileChannel = new FileInputStream(file1).getChannel();
+        try
+        {
+            // test drain 0
+            assertEquals(0, Channels.drain(fileChannel, 0));
+            ByteBuffer buffer = ByteBuffer.allocate(10);
+            fileChannel.read(buffer, 0);
+            assertReadMessage(buffer, "read data\n");
+
+            // test drain negative
+            buffer.clear();
+            boolean failed = false;
+            try {
+                assertEquals(0, Channels.drain(fileChannel, 10, -3));
+            } catch (IllegalArgumentException illegalArgumentException) {
+                failed = true;
+            }
+            assertTrue(failed);
+            assertEquals(10, fileChannel.read(buffer, 10));
+            assertReadMessage(buffer, "read data\n");
+
+            // test drain 2
+            assertEquals(2, Channels.drain(fileChannel, 20,2));
+            buffer.clear();
+            buffer.limit(8);
+            assertEquals(8, fileChannel.read(buffer, 22));
+            assertReadMessage(buffer, "ad data\n");
+
+            // test drain little by little
+            buffer.clear();
+            buffer.limit(4);
+            assertEquals(1, Channels.drain(fileChannel, 30, 1));
+            assertEquals(2, Channels.drain(fileChannel, 31,2));
+            assertEquals(3, Channels.drain(fileChannel, 33,3));
+            assertEquals(4, fileChannel.read(buffer, 36));
+            assertReadMessage(buffer, "ata\n");
+
+            // test drain more bytes than available
+            buffer.clear();
+            assertEquals(10, Channels.drain(fileChannel, 40, 11));
+
+            // test drain an already drained channel
+            assertEquals(-1, Channels.drain(fileChannel, 50, Long.MAX_VALUE));
+            assertEquals(-1, fileChannel.read(buffer, 50));
+        } finally {
+            fileChannel.close();
+        }
+        boolean failed = false;
+        try {
+            Channels.drain(fileChannel, Long.MAX_VALUE);
+        } catch (ClosedChannelException e) {
+            failed = true;
+        }
+        assertTrue(failed);
+    }
+
+    private void assertDrain(ReadableByteChannelMock channelMock, LongFunction<Long> drainFunction) throws IOException {
+        channelMock.enableClosedCheck(true);
+        // test drain 0
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        assertEquals(0L, (long) drainFunction.apply(0));
+        ByteBuffer buffer = ByteBuffer.allocate(10);
+        channelMock.read(buffer);
+        assertReadMessage(buffer, "read", "data");
+
+        // test drain negative
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        buffer.clear();
+        boolean failed = false;
+        try {
+            drainFunction.apply(-5);
+        } catch (IllegalArgumentException illegalArgumentException) {
+            failed = true;
+        }
+        assertTrue(failed);
+        assertEquals(8, channelMock.read(buffer));
+        assertReadMessage(buffer, "read", "data");
+
+        // test drain 2
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        assertEquals(2L, (long) drainFunction.apply(2));
+        buffer.clear();
+        assertEquals(6, channelMock.read(buffer));
+        assertReadMessage(buffer, "ad", "data");
+
+        // test drain little by little
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        buffer.clear();
+        assertEquals(1L, (long) drainFunction.apply(1));
+        assertEquals(2L, (long) drainFunction.apply(2));
+        assertEquals(3L, (long) drainFunction.apply(3));
+        assertEquals(2, channelMock.read(buffer));
+        assertReadMessage(buffer, "ta");
+
+        // test drain the exact amount of bytes left
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        buffer.clear();
+        assertEquals(8L, (long) drainFunction.apply(8));
+        assertEquals(0, channelMock.read(buffer));
+
+        // test drain more bytes than available
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        buffer.clear();
+        assertEquals(8L, (long) drainFunction.apply(9));
+        assertEquals(0, channelMock.read(buffer));
+
+        // test drain the exact amount of bytes left without reading the EOF
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        channelMock.setEof();
+        buffer.clear();
+        assertEquals(8L, (long) drainFunction.apply(8));
+        assertEquals(-1, channelMock.read(buffer));
+
+        // test drain more bytes than available with eof
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        buffer.clear();
+        assertEquals(8L, (long) drainFunction.apply(9));
+        assertEquals(-1, channelMock.read(buffer));
+
+        // test drain with long max (Undertow usage)
+        channelMock.setReadData("read", "data");
+        channelMock.enableRead(true);
+        buffer.clear();
+        assertEquals(8L, (long) drainFunction.apply(Long.MAX_VALUE));
+        assertEquals(-1, channelMock.read(buffer));
+
+        // test drain an already drained channel
+        assertEquals(-1L, (long) drainFunction.apply(Long.MAX_VALUE));
+        assertEquals(-1, channelMock.read(buffer));
+
+        channelMock.close();
+        failed = false;
+        try {
+            drainFunction.apply(Long.MAX_VALUE);
+        } catch (RuntimeException e) {
+            assertTrue(e.getCause() instanceof ClosedChannelException);
+            failed = true;
+        }
+        assertTrue(failed);
+    }
+
     public static class FlushBlocking implements Runnable {
         private final SuspendableWriteChannel channel;
 
@@ -765,7 +974,7 @@ public class ChannelsTestCase {
         public void run() {
             ByteBuffer buffer = ByteBuffer.allocate(30);
             try {
-                buffer.put(message.getBytes("UTF-8")).flip();
+                buffer.put(message.getBytes(UTF_8)).flip();
                 if (timeoutUnit != null) {
                     writeResult = Channels.writeBlocking(channel, buffer, timeout, timeoutUnit);
                 } else {
@@ -805,7 +1014,7 @@ public class ChannelsTestCase {
             try {
                 for (int i = 0; i < buffer.length; i++) {
                     buffer[i] = ByteBuffer.allocate(message[i].length());
-                    buffer[i].put(message[i].getBytes("UTF-8")).flip();
+                    buffer[i].put(message[i].getBytes(UTF_8)).flip();
                 }
                 if (timeoutUnit != null) {
                     writeResult = Channels.writeBlocking(channel, buffer, 0, buffer.length, timeout, timeoutUnit);
@@ -844,7 +1053,7 @@ public class ChannelsTestCase {
         public void run() {
             ByteBuffer buffer = ByteBuffer.allocate(30);
             try {
-                buffer.put(message.getBytes("UTF-8")).flip();
+                buffer.put(message.getBytes(UTF_8)).flip();
                 if (timeoutUnit != null) {
                     sendResult = Channels.sendBlocking(channel, buffer, timeout, timeoutUnit);
                 } else {
@@ -884,7 +1093,7 @@ public class ChannelsTestCase {
             try {
                 for (int i = 0; i < buffer.length; i++) {
                     buffer[i] = ByteBuffer.allocate(message[i].length());
-                    buffer[i].put(message[i].getBytes("UTF-8")).flip();
+                    buffer[i].put(message[i].getBytes(UTF_8)).flip();
                 }
                 if (timeoutUnit != null) {
                     sendResult = Channels.sendBlocking(channel, buffer, 0, buffer.length, timeout, timeoutUnit);
