@@ -20,6 +20,7 @@
 package org.xnio.ssl.mock;
 
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
 import java.util.HashMap;
 import java.util.Map;
 
@@ -30,6 +31,7 @@ import javax.net.ssl.SSLEngineResult.Status;
 import javax.net.ssl.SSLException;
 import javax.net.ssl.SSLSession;
 
+import org.jboss.logging.Logger;
 import org.jmock.Expectations;
 import org.jmock.Mockery;
 import org.xnio.Buffers;
@@ -68,6 +70,8 @@ import org.xnio.Buffers;
  *
  */
 public class SSLEngineMock extends SSLEngine {
+    private static final Logger log = Logger.getLogger("TEST");
+
     // every wrapped handshake message generated by this engine is the result of wrapping HANDSHAKE_MSG
     public static final String HANDSHAKE_MSG = "[handshake data]";
     // every wrapped close message generated by this engine is the resujlt of wrapping CLOSE_MSG
@@ -187,7 +191,7 @@ public class SSLEngineMock extends SSLEngine {
      * Marks this engine as closed.
      */
     @Override
-    public void closeInbound() throws SSLException {
+    public synchronized void closeInbound() throws SSLException {
         closed = true;
     }
 
@@ -195,7 +199,7 @@ public class SSLEngineMock extends SSLEngine {
      * Marks this engine as closed.
      */
     @Override
-    public void closeOutbound() {
+    public synchronized void closeOutbound() {
         closed = true;
     }
 
@@ -213,6 +217,7 @@ public class SSLEngineMock extends SSLEngine {
      * </ul>
      * 
      */
+    @SuppressWarnings("incomplete-switch")
     @Override
     public synchronized Runnable getDelegatedTask() {
         if (!closed && actionIndex < actions.length) {
@@ -280,7 +285,7 @@ public class SSLEngineMock extends SSLEngine {
     }
 
     @Override
-    public HandshakeStatus getHandshakeStatus() {
+    public synchronized HandshakeStatus getHandshakeStatus() {
         final int currentIndex;
         synchronized (this) {
             if (closed) {
@@ -299,6 +304,7 @@ public class SSLEngineMock extends SSLEngine {
         if(currentIndex >= actions.length) {
             return HandshakeStatus.NOT_HANDSHAKING;
         }
+        log.debug("Current engine mock action: " + actions[currentIndex]);
         switch(actions[currentIndex]) {
             case NEED_TASK:
             case NEED_FAULTY_TASK:
@@ -316,18 +322,32 @@ public class SSLEngineMock extends SSLEngine {
         }
     }
 
+    private SSLEngineResult logOperation(String operation, SSLEngineResult result) {
+        log.debugf("SSLEngineMock.%s returned [%s, %s, consumed:%d, produced:%d]", operation, result.getStatus(),
+                result.getHandshakeStatus(), result.bytesConsumed(), result.bytesProduced());
+        return result;
+    }
+
+    private SSLEngineResult logUnwrap(SSLEngineResult result) {
+        return logOperation("unwrap", result);
+    }
+
+    private SSLEngineResult logWrap(SSLEngineResult result) {
+        return logOperation("wrap", result);
+    }
+
     @Override
-    public SSLEngineResult unwrap(ByteBuffer src, ByteBuffer[] dsts, int offset, int length) throws SSLException {
+    public synchronized SSLEngineResult unwrap(ByteBuffer src, ByteBuffer[] dsts, int offset, int length) throws SSLException {
         for (int i = offset; i < length; i++) {
             if (dsts[i].hasRemaining()) {
                 break;
             }
             else if (i == length -1) {
-                return new SSLEngineResult(SSLEngineResult.Status.BUFFER_OVERFLOW, getHandshakeStatus(), 0, 0);
+                return logUnwrap(new SSLEngineResult(SSLEngineResult.Status.BUFFER_OVERFLOW, getHandshakeStatus(), 0, 0));
             }
         }
         if (closed && closeMessageUnwrapped) {
-            return new SSLEngineResult(Status.CLOSED, HandshakeStatus.NOT_HANDSHAKING, 0, 0);
+            return logUnwrap(new SSLEngineResult(Status.CLOSED, HandshakeStatus.NOT_HANDSHAKING, 0, 0));
         }
         final HandshakeStatus status;
         final int currentActionIndex;
@@ -335,30 +355,40 @@ public class SSLEngineMock extends SSLEngine {
             status = getHandshakeStatus();
             currentActionIndex = actionIndex;
         }
+        final SSLEngineResult result;
         switch(status) {
             case FINISHED:
                 actionAccountedFor(HandshakeAction.FINISH, currentActionIndex);
-                return new SSLEngineResult(Status.OK, HandshakeStatus.FINISHED, 30, 0);
+                result = new SSLEngineResult(Status.OK, HandshakeStatus.FINISHED, 30, 0);
+                break;
             case NEED_TASK:
-                return new SSLEngineResult(Status.OK, HandshakeStatus.NEED_TASK, 0, 0);
+                result = new SSLEngineResult(Status.OK, HandshakeStatus.NEED_TASK, 0, 0);
+                break;
             case NEED_UNWRAP:
-                return wrapper.unwrap(dsts, offset, length, src, true, currentActionIndex);
+                result = wrapper.unwrap(dsts, offset, length, src, true, currentActionIndex);
+                break;
             case NEED_WRAP:
                 if (closed) {
-                    return new SSLEngineResult(Status.CLOSED, SSLEngineResult.HandshakeStatus.NEED_WRAP, 0, 0);
+                    result = new SSLEngineResult(Status.CLOSED, SSLEngineResult.HandshakeStatus.NEED_WRAP, 0, 0);
+                } else {
+                    result =  new SSLEngineResult(Status.OK, SSLEngineResult.HandshakeStatus.NEED_WRAP, 0, 0);
                 }
-                return new SSLEngineResult(Status.OK, SSLEngineResult.HandshakeStatus.NEED_WRAP, 0, 0);
+                break;
             case NOT_HANDSHAKING: 
-                return wrapper.unwrap(dsts, offset, length, src, false, currentActionIndex);
+                actionAccountedFor(HandshakeAction.PERFORM_REQUESTED_ACTION, currentActionIndex);
+                result = wrapper.unwrap(dsts, offset, length, src, false, currentActionIndex);
+                break;
             default:
                 throw new IllegalStateException("Unexpected handshake status: " + getHandshakeStatus());
         }
+        logUnwrap(result);
+        return result;
     }
 
     @Override
-    public SSLEngineResult wrap(ByteBuffer[] srcs, int offset, int length, ByteBuffer dst) throws SSLException {
+    public synchronized SSLEngineResult wrap(ByteBuffer[] srcs, int offset, int length, ByteBuffer dst) throws SSLException {
         if (dst.position() > 0) {
-            return new SSLEngineResult(SSLEngineResult.Status.BUFFER_OVERFLOW, getHandshakeStatus(), 0, 0);
+            return logWrap(new SSLEngineResult(SSLEngineResult.Status.BUFFER_OVERFLOW, getHandshakeStatus(), 0, 0));
         }
         final HandshakeStatus status;
         final int currentActionIndex;
@@ -366,23 +396,35 @@ public class SSLEngineMock extends SSLEngine {
             status = getHandshakeStatus();
             currentActionIndex = actionIndex;
         }
+        final SSLEngineResult result;
         switch(status) {
             case FINISHED: {
                 actionAccountedFor(HandshakeAction.FINISH, currentActionIndex);
-                return new SSLEngineResult(SSLEngineResult.Status.OK, SSLEngineResult.HandshakeStatus.FINISHED, 0, 0);
+                result = new SSLEngineResult(SSLEngineResult.Status.OK, SSLEngineResult.HandshakeStatus.FINISHED, 0, 0);
+                break;
             }
             case NEED_TASK:
-                return new SSLEngineResult(SSLEngineResult.Status.OK, SSLEngineResult.HandshakeStatus.NEED_TASK, 0, 0);
+                result = new SSLEngineResult(SSLEngineResult.Status.OK, SSLEngineResult.HandshakeStatus.NEED_TASK, 0, 0);
+                break;
             case NEED_UNWRAP:
-                return new SSLEngineResult(SSLEngineResult.Status.OK, SSLEngineResult.HandshakeStatus.NEED_UNWRAP, 0, 0);
+                result = new SSLEngineResult(closed? SSLEngineResult.Status.CLOSED: SSLEngineResult.Status.OK, SSLEngineResult.HandshakeStatus.NEED_UNWRAP, 0, 0);
+                break;
             case NEED_WRAP:
-                return wrapper.wrap(dst, srcs, offset, length, true, currentActionIndex);
+                result = wrapper.wrap(dst, srcs, offset, length, true, currentActionIndex);
+                break;
             case NOT_HANDSHAKING: {
-                return wrapper.wrap(dst, srcs, offset, length, false, currentActionIndex);
+                if (closed) {
+                    return new SSLEngineResult(SSLEngineResult.Status.CLOSED, SSLEngineResult.HandshakeStatus.NOT_HANDSHAKING, 0, 0);
+                }
+                actionAccountedFor(HandshakeAction.PERFORM_REQUESTED_ACTION, currentActionIndex);
+                result = wrapper.wrap(dst, srcs, offset, length, false, currentActionIndex);
+                break;
             }
             default:
                 throw new IllegalStateException("Unexpected handshake status: " + getHandshakeStatus());
         }
+        logWrap(result);
+        return result;
     }
 
     private final class MappedWrapper {
@@ -403,16 +445,9 @@ public class SSLEngineMock extends SSLEngine {
             }
         }
 
-        private boolean hasEnoughSpace(ByteBuffer[] dsts, int offset, int length, int dataLength) {
-            for (int i = offset; i < length && dataLength > 0; i++) {
-                dataLength -= dsts[i].remaining();
-            }
-            return dataLength <= 0;
-        }
-
         public SSLEngineResult unwrap(ByteBuffer[] dsts, int offset, int length, ByteBuffer src, boolean needUnwrap,int actionIndex) {
             if (!src.hasRemaining()) {
-                return new SSLEngineResult(closed && closeMessageUnwrapped? Status.CLOSED: Status.BUFFER_UNDERFLOW, HandshakeStatus.NEED_UNWRAP, 0, 0);
+                return new SSLEngineResult(closed && closeMessageUnwrapped? Status.CLOSED: Status.BUFFER_UNDERFLOW, needUnwrap? HandshakeStatus.NEED_UNWRAP: HandshakeStatus.NOT_HANDSHAKING, 0, 0);
             }
             Status okStatus = closed && closeMessageUnwrapped? Status.CLOSED: Status.OK;
             // amount of bytes available at src
@@ -457,7 +492,7 @@ public class SSLEngineMock extends SSLEngine {
 
         private String unwrapBytes(ByteBuffer[] dsts, int offset, int length, ByteBuffer src, boolean readHandshakeMsg) {
             // define unwrapped data to be written to dsts
-            String wrapped = Buffers.getModifiedUtf8(src);
+            String wrapped = new String(Buffers.take(src), StandardCharsets.ISO_8859_1);
             int wrappedEndIndex = wrapped.length();
             int wrappedLeftOver = -1;
             while (wrappedEndIndex > 0 && !unwrapMap.containsKey(wrapped.substring(0, wrappedEndIndex))) {
@@ -501,26 +536,12 @@ public class SSLEngineMock extends SSLEngine {
                     return null;
                 }
                 // check if there is enough space to write unwrapped data, if not, do not write
-                if (!hasEnoughSpace(dsts, offset, length, unwrapped.length())) {
+                if (Buffers.remaining(dsts, offset, length) < unwrapped.length()) {
                     src.position(src.position() - wrapped.length());
                     return null;
                 }
                 // copy data to dsts
-                int unwrappedSliceIndex = 0;
-                int dstsLength = offset + length;
-                for (int i = offset; i < dstsLength; i++) {
-                    String unwrappedData = unwrapped;
-                    boolean done = true;
-                    if (dsts[i].remaining() < unwrapped.length()) {
-                        unwrappedData = unwrapped.substring(unwrappedSliceIndex, dsts[i].remaining());
-                        unwrappedSliceIndex += dsts[i].remaining();
-                        done = false;
-                    }
-                    Buffers.putModifiedUtf8(dsts[i], unwrappedData);
-                    if (done) {
-                        break;
-                    }
-                }
+                Buffers.copy(dsts, offset, length, ByteBuffer.wrap(unwrapped.getBytes(StandardCharsets.ISO_8859_1)));
             }
             return unwrapped;
         }
