@@ -294,146 +294,4 @@ public final class NioFullDuplexChannelPipeTestCase extends AbstractNioChannelPi
         leftChannelOK.set(true);
         rightChannelOK.set(true);
     }
-
-    @Test
-    public void leftChannelNastyClose() throws Exception {
-        log.info("Test: leftChannelNastyClose");
-        final CountDownLatch latch = new CountDownLatch(2);
-        doConnectionTest(new LatchAwaiter(latch), new ChannelListener<StreamChannel>() {
-            public void handleEvent(final StreamChannel channel) {
-                try {
-                    channel.getCloseSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    channel.setOption(Options.CLOSE_ABORT, Boolean.TRUE);
-                    channel.close();
-                    leftChannelOK.set(true);
-                } catch (Throwable t) {
-                    log.errorf(t, "Failed to close channel (propagating as RT exception)");
-                    latch.countDown();
-                    throw new RuntimeException(t);
-                }
-            }
-        }, new ChannelListener<StreamChannel>() {
-            public void handleEvent(final StreamChannel channel) {
-                try {
-                    channel.getCloseSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    channel.getReadSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            int res;
-                            try {
-                                res = channel.read(ByteBuffer.allocate(100));
-                            } catch (IOException e) {
-                                IoUtils.safeClose(channel);
-                                return;
-                            }
-                            if (res == -1) {
-                                rightChannelOK.set(true);
-                                IoUtils.safeClose(channel);
-                            }
-                        }
-                    });
-                    channel.resumeReads();
-                } catch (Throwable t) {
-                    log.errorf(t, "Failed to close channel (propagating as RT exception)");
-                    latch.countDown();
-                    throw new RuntimeException(t);
-                }
-            }
-        });
-    }
-
-    @Test
-    public void rightChannelNastyClose() throws Exception {
-        log.info("Test: rightChannelNastyClose");
-        final CountDownLatch latch = new CountDownLatch(2);
-        final CountDownLatch rightChannelLatch = new CountDownLatch(1);
-        doConnectionTest(new LatchAwaiter(latch), new ChannelListener<StreamChannel>() {
-            public void handleEvent(final StreamChannel channel) {
-                try {
-                    log.info("Left channel opened");
-                    channel.getCloseSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    channel.getReadSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            int res;
-                            try {
-                                res = channel.read(ByteBuffer.allocate(100));
-                            } catch (IOException e) {
-                                IoUtils.safeClose(channel);
-                                return;
-                            }
-                            if (res == -1) {
-                                leftChannelOK.set(true);
-                                IoUtils.safeClose(channel);
-                            }
-                        }
-                    });
-                    channel.getWriteSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            try {
-                                if (channel.write(ByteBuffer.wrap(new byte[] { 1 })) > 0) {
-                                    channel.suspendWrites();
-                                }
-                            } catch (IOException e) {
-                                IoUtils.safeClose(channel);
-                            }
-                        }
-                    });
-                    channel.resumeReads();
-                    channel.resumeWrites();
-                    rightChannelLatch.countDown();
-                } catch (Throwable t) {
-                    log.error("Error occurred on client", t);
-                    try {
-                        channel.close();
-                    } catch (Throwable t2) {
-                        log.error("Error occurred on client (close)", t2);
-                        latch.countDown();
-                        throw new RuntimeException(t);
-                    }
-                    throw new RuntimeException(t);
-                }
-            }
-        }, new ChannelListener<StreamChannel>() {
-            public void handleEvent(final StreamChannel channel) {
-                try {
-                    log.info("Right channel opened");
-                    channel.getCloseSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    channel.getReadSetter().set(new ChannelListener<StreamChannel>() {
-                        public void handleEvent(final StreamChannel channel) {
-                            try {
-                                if (channel.read(ByteBuffer.allocate(1)) > 0) {
-                                    log.info("Closing connection...");
-                                    channel.setOption(Options.CLOSE_ABORT, Boolean.TRUE);
-                                    channel.close();
-                                    rightChannelOK.set(true);
-                                }
-                            } catch (IOException e) {
-                                IoUtils.safeClose(channel);
-                            }
-                        }
-                    });
-                    channel.resumeReads();
-                } catch (Throwable t) {
-                    log.error("Error occurred on server", t);
-                    latch.countDown();
-                    throw new RuntimeException(t);
-                }
-            }
-        });
-    }
 }
